
#include "Board.h"
#include "interrupt_timer.h"

volatile unsigned int uiNextInterruptTimer0;	//  ͷƮ ɸ ð

#define TIMER0_INTERRUPT_LEVEL		1
//#define TIMER1_INTERRUPT_LEVEL		4

/*-----------------*/
/* Clock Selection */
/*-----------------*/
#define TC_CLKS                  0x7
#define TC_CLKS_MCK2             0x0
#define TC_CLKS_MCK8             0x1
#define TC_CLKS_MCK32            0x2
#define TC_CLKS_MCK128           0x3
#define TC_CLKS_MCK1024          0x4

// 	AT91F_TC_Open(AT91C_BASE_TC0,TC_CLKS_MCK1024,AT91C_ID_TC0);

//*------------------------- Internal Function --------------------------------
//*----------------------------------------------------------------------------
//* Function Name       : AT91F_TC_Open
//* Object              : Initialize Timer Counter Channel and enable is clock
//* Input Parameters    : <tc_pt> = TC Channel Descriptor Pointer
//*                       <mode> = Timer Counter Mode
//*                     : <TimerId> = Timer peripheral ID definitions
//* Output Parameters   : None
//*----------------------------------------------------------------------------
void AT91F_TC_Open ( AT91PS_TC TC_pt, unsigned int Mode, unsigned int TimerId)
{
    unsigned int dummy;

    //  Ÿ̸Ӹ ENABLE Ѵ.
   	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1<< TimerId ) ;

    // ϴ Ÿ̸ӿ ͷƮ Ų.
	TC_pt->TC_CCR = AT91C_TC_CLKDIS ;
	TC_pt->TC_IDR = 0xFFFFFFFF ;

    //* Clear status bit
	dummy = TC_pt->TC_SR;
    dummy = dummy; // variable "dummy" was set but never used   
        
    //* Set the Mode of the Timer Counter
	TC_pt->TC_CMR = Mode ;

    //* Enable the clock
	TC_pt->TC_CCR = AT91C_TC_CLKEN ;

}

#define COMPARE0_500US_WHEN_TC_CLKS_MCK2	12013 // =   1sec/(1000/48.054857 * MCK2)/2
#define COMPARE0_1MS_WHEN_TC_CLKS_MCK2		24027 // =   1sec/(1000/48.054857 * MCK2)


//*----------------------------------------------------------------------------
//* Function Name       : timer0_c_irq_handler
//* Object              : C handler interrupt function called by the interrupts
//*                       assembling routine
//* Output Parameters   : increment count_timer0_interrupt
//*----------------------------------------------------------------------------
void timer0_c_irq_handler(void)
{
	AT91PS_TC TC_pt = AT91C_BASE_TC0;
	unsigned int uiTimerStatus = TC_pt->TC_SR;

	if (AT91C_TC_CPAS == (uiTimerStatus&AT91C_TC_CPAS))
	{
		//* Read the output state
		if ( (AT91F_PIO_GetInput(AT91C_BASE_PIOA) & LED_OPER ) == LED_OPER )
		{
			AT91F_PIO_ClearOutput(AT91C_BASE_PIOA, LED_OPER );
		}
		else
		{
			AT91F_PIO_SetOutput(AT91C_BASE_PIOA, LED_OPER );
		}
		uiNextInterruptTimer0 +=COMPARE0_1MS_WHEN_TC_CLKS_MCK2;
		AT91C_BASE_TC0->TC_RA = uiNextInterruptTimer0;	
	}
}

void timer_init ( void )
{

    //* Open timer0
	// Waveform Operating Mode
	AT91F_TC_Open(AT91C_BASE_TC0,TC_CLKS_MCK2|AT91C_TC_WAVE,AT91C_ID_TC0);

    //* Open Timer 0 interrupt
	AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_TC0, TIMER0_INTERRUPT_LEVEL,AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, timer0_c_irq_handler);
	AT91C_BASE_TC0->TC_IER = AT91C_TC_CPAS | AT91C_TC_COVFS;  //  IRQ enable CPC
	AT91C_BASE_TC0->TC_RA = 10000;	

	AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_TC0);

	// Generate interrupt by software
	AT91F_AIC_Trig(AT91C_BASE_AIC,AT91C_ID_TC0);

    // Start timer0
	AT91C_BASE_TC0->TC_CCR = AT91C_TC_SWTRG ;
}
